/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import axios from 'axios';

export function Delete(api) {
    return new Promise(resolve => {
        axios({
            method: 'delete',
            url: api,
            headers: {
                'Accept': 'application/json',
                'Content-Type': 'application/json; charset=utf-8',
                'access-token': sessionStorage.getItem('@portalmec/accessToken'),
                'client': sessionStorage.getItem('@portalmec/clientToken'),
                'uid': sessionStorage.getItem('@portalmec/uid'),
                'If-None-Match': null
            },
        }).then(response => {
            if (response.status === 200) {
                resolve(true);
            } else {
                resolve(false);
            }
            SaveNewHeaders(response)
        })
    })
}

export function SendEmail(api, body) {
    return new Promise(resolve => {
        axios({
            method: 'post',
            url: api,
            headers: {
                'Accept': 'application/json',
                'Content-Type': 'application/json; charset=utf-8',
                'access-token': sessionStorage.getItem('@portalmec/accessToken'),
                'client': sessionStorage.getItem('@portalmec/clientToken'),
                'uid': sessionStorage.getItem('@portalmec/uid'),
                'If-None-Match': null
            },
            data: JSON.stringify(body)
        }).then(response => {
            console.log(response)
            if (response.status === 200) {
                resolve(true);
            } else {
                resolve(false);
            }
            SaveNewHeaders(response)
        })
    })
}

export function Edit(api, body) {
    return new Promise(resolve => {
        axios({
            method: 'put',
            url: api,
            headers: {
                'Accept': 'application/json',
                'Content-Type': 'application/json; charset=utf-8',
                'access-token': sessionStorage.getItem('@portalmec/accessToken'),
                'client': sessionStorage.getItem('@portalmec/clientToken'),
                'uid': sessionStorage.getItem('@portalmec/uid'),
                'If-None-Match': null
            },
            data: JSON.stringify(body)
        }).then(response => {
            console.log(response)
            if (response.status === 200) {
                resolve(true)
            } else {
                resolve(false)
            }
            SaveNewHeaders(response)
        })
    })
}

export function Create(api, body) {
    return new Promise(resolve => {
        axios({
            method: 'post',
            url: api,
            headers: {
                'Accept': 'application/json',
                'Content-Type': 'application/json; charset=utf-8',
                'access-token': sessionStorage.getItem('@portalmec/accessToken'),
                'client': sessionStorage.getItem('@portalmec/clientToken'),
                'uid': sessionStorage.getItem('@portalmec/uid'),
                'If-None-Match': null
            },
            data: JSON.stringify(body)
        }).then(response => {
            if (response.status === 201) {
                resolve(true)
            } else {
                resolve(false)
            }
            SaveNewHeaders(response)
        })
    })
}

export function GetFullList(api, headers) {
    return new Promise(resolve => {
        axios({
            method: 'get',
            url: api,
            headers: headers
        }).then((res) => {
            console.log(res)
            if (res.status === 200) {
                resolve({
                    state: true,
                    data: res.data
                })
            } else {
                resolve({
                    state: false,
                    data: {}
                })
            }
            if (headers !== undefined) {
                SaveNewHeaders(res)
            }
        }).catch((err) => {
            resolve({
                state: false,
                data: {}
            })
        })
    }, []);
}

const SaveNewHeaders = (response) => {
    if (
        (response.headers['access-token'] === undefined || response.headers['access-token'] === null) &&
        (response.headers.client === undefined || response.headers.client === null)
    ) {

    } else {
        sessionStorage.setItem('@portalmec/accessToken', response.headers['access-token'])
        sessionStorage.setItem('@portalmec/clientToken', response.headers.client)
        console.log('saved')
    }
}

